@@ -7,7 +7,6 @@ module Debian.Debianize.Goodies
     , doWebsite
     , doBackups
     , doExecutable
-    , debianDescription
     , describe
     , watchAtom
     , oldClckwrksSiteFlags
@@ -16,33 +15,32 @@ module Debian.Debianize.Goodies
     , serverAtoms
     , backupAtoms
     , execAtoms
+    , makeRulesHead
     ) where
 
-import Data.Lens.Lazy (getL, modL)
+import Data.Char (toLower)
+import Data.Lens.Lazy (modL, access)
 import Data.List as List (map, intersperse, intercalate)
 import Data.Map as Map (insertWith)
 import Data.Maybe (fromMaybe)
 import Data.Monoid ((<>))
 import Data.Set as Set (insert, union, singleton)
 import Data.Text as Text (Text, pack, unlines, intercalate)
-import Debian.Debianize.Atoms as Atoms ()
-import Debian.Debianize.Atoms as Atoms
-    (Atoms, packageDescription, rulesFragments, website, serverInfo, link, backups, executable,
-     install, installTo, installCabalExecTo, file, installDir, logrotateStanza, postInst,
-     installInit, installCabalExec, rulesFragments, packageDescription, executable,
-     serverInfo, website, backups, depends)
-import Debian.Debianize.ControlFile as Debian (PackageType(..))
-import Debian.Debianize.Types (InstallFile(..), Server(..), Site(..))
-import Debian.Debianize.Utility (trim)
+import Debian.Debianize.DebianName (debianName)
+import Debian.Debianize.Monad (Atoms, DebT, execDebM)
+import Debian.Debianize.Prelude (trim, (%=), (+=), (++=), (+++=))
+import qualified Debian.Debianize.Types as T
+import qualified Debian.Debianize.Types.Atoms as T
+import qualified Debian.Debianize.Types.BinaryDebDescription as B
 import Debian.Orphans ()
+import Debian.Pretty (Pretty(pretty))
 import Debian.Policy (apacheLogDirectory, apacheErrorLog, apacheAccessLog, databaseDirectory, serverAppLog, serverAccessLog)
 import Debian.Relation (BinPkgName(BinPkgName), Relation(Rel))
 import Distribution.Package (PackageIdentifier(..), PackageName(PackageName))
-import qualified Distribution.PackageDescription as Cabal
+import Distribution.PackageDescription as Cabal (PackageDescription(package, synopsis, description, author, maintainer, pkgUrl))
 import Distribution.Text (display)
 import Prelude hiding (writeFile, init, unlines, log, map)
 import System.FilePath ((</>))
-import Text.PrettyPrint.ANSI.Leijen (Pretty(pretty))
 
 showCommand :: String -> [String] -> String
 showCommand cmd args =
@@ -59,19 +57,18 @@ translate str =
 -- to find out B's version number, version B.  Then write a rule into
 -- P's .substvar that makes P require that that exact version of A,
 -- and another that makes P conflict with any older version of A.
-tightDependencyFixup :: [(BinPkgName, BinPkgName)] -> BinPkgName -> Atoms -> Atoms
-tightDependencyFixup [] _ deb = deb
-tightDependencyFixup pairs p deb =
-    modL rulesFragments
-             (Set.insert
-              (Text.unlines $
+tightDependencyFixup :: Monad m => [(BinPkgName, BinPkgName)] -> BinPkgName -> DebT m ()
+tightDependencyFixup [] _ = return ()
+tightDependencyFixup pairs p =
+    T.rulesFragments +=
+          (Text.unlines $
                ([ "binary-fixup/" <> name <> "::"
                 , "\techo -n 'haskell:Depends=' >> debian/" <> name <> ".substvars" ] ++
                 intersperse ("\techo -n ', ' >> debian/" <> name <> ".substvars") (List.map equals pairs) ++
                 [ "\techo '' >> debian/" <> name <> ".substvars"
                 , "\techo -n 'haskell:Conflicts=' >> debian/" <> name <> ".substvars" ] ++
                 intersperse ("\techo -n ', ' >> debian/" <> name <> ".substvars") (List.map newer pairs) ++
-                [ "\techo '' >> debian/" <> name <> ".substvars" ]))) deb
+                [ "\techo '' >> debian/" <> name <> ".substvars" ]))
     where
       equals (installed, dependent) = "\tdpkg-query -W -f='" <> display' dependent <> " (=$${Version})' " <>  display' installed <> " >> debian/" <> name <> ".substvars"
       newer  (installed, dependent) = "\tdpkg-query -W -f='" <> display' dependent <> " (>>$${Version})' " <> display' installed <> " >> debian/" <> name <> ".substvars"
@@ -79,95 +76,92 @@ tightDependencyFixup pairs p deb =
       display' = pack . show . pretty
 
 -- | Add a debian binary package to the debianization containing a cabal executable file.
-doExecutable :: BinPkgName -> InstallFile -> Atoms -> Atoms
-doExecutable bin x deb = modL executable (Map.insertWith (\ a b -> error $ "doExecutable: " ++ show (a, b)) bin x) deb
+doExecutable :: Monad m => BinPkgName -> T.InstallFile -> DebT m ()
+doExecutable p f = T.executable ++= (p, f)
 
 -- | Add a debian binary package to the debianization containing a cabal executable file set up to be a server.
-doServer :: BinPkgName -> Server -> Atoms -> Atoms
-doServer bin x deb = modL serverInfo (Map.insertWith (\ a b -> error $ "doServer: " ++ show (a, b)) bin x) deb
+doServer :: Monad m => BinPkgName -> T.Server -> DebT m ()
+doServer p s = T.serverInfo ++= (p, s)
 
 -- | Add a debian binary package to the debianization containing a cabal executable file set up to be a web site.
-doWebsite :: BinPkgName -> Site -> Atoms -> Atoms
-doWebsite bin x deb = modL website (Map.insertWith (\ a b -> error $ "doWebsite: " ++ show (a, b)) bin x) deb
+doWebsite :: Monad m => BinPkgName -> T.Site -> DebT m ()
+doWebsite p w = T.website ++= (p, w)
 
 -- | Add a debian binary package to the debianization containing a cabal executable file set up to be a backup script.
-doBackups :: BinPkgName -> String -> Atoms -> Atoms
-doBackups bin s deb =
-    modL backups (Map.insertWith (error "backups") bin s) $
-    modL Atoms.depends (Map.insertWith union bin (singleton (Rel (BinPkgName "anacron") Nothing Nothing))) $
-    deb
+doBackups :: Monad m => BinPkgName -> String -> DebT m ()
+doBackups bin s =
+    do T.backups ++= (bin, s)
+       T.depends bin %= (++ [[Rel (BinPkgName "anacron") Nothing Nothing]])
+       -- depends +++= (bin, Rel (BinPkgName "anacron") Nothing Nothing)
 
-describe :: Atoms -> PackageType -> PackageIdentifier -> Text
-describe atoms typ ident =
-    debianDescription (Cabal.synopsis pkgDesc) (Cabal.description pkgDesc) (Cabal.author pkgDesc) (Cabal.maintainer pkgDesc) (Cabal.pkgUrl pkgDesc) typ ident
-    where
-      pkgDesc = fromMaybe (error $ "describe " ++ show ident) $ getL packageDescription atoms
-
-debianDescription :: String -> String -> String -> String -> String -> PackageType -> PackageIdentifier -> Text
-debianDescription synopsis' description' author' maintainer' url typ pkgId =
-    debianDescriptionBase synopsis' description' author' maintainer' url <> "\n" <>
-    case typ of
-      Profiling ->
-          Text.intercalate "\n"
-                  [" .",
-                   " This package provides a library for the Haskell programming language, compiled",
-                   " for profiling.  See http:///www.haskell.org/ for more information on Haskell."]
-      Development ->
-          Text.intercalate "\n"
-                  [" .",
-                   " This package provides a library for the Haskell programming language.",
-                   " See http:///www.haskell.org/ for more information on Haskell."]
-      Documentation ->
-          Text.intercalate "\n"
-                  [" .",
-                   " This package provides the documentation for a library for the Haskell",
-                   " programming language.",
-                   " See http:///www.haskell.org/ for more information on Haskell." ]
-      Exec ->
-          Text.intercalate "\n"
-                  [" .",
-                   " An executable built from the " <> pack (display (pkgName pkgId)) <> " package."]
-{-    ServerPackage ->
-          Text.intercalate "\n"
-                  [" .",
-                   " A server built from the " <> pack (display (pkgName pkgId)) <> " package."] -}
-      Utilities ->
-          Text.intercalate "\n"
-                  [" .",
-                   " Utility files associated with the " <> pack (display (pkgName pkgId)) <> " package."]
-      x -> error $ "Unexpected library package name suffix: " ++ show x
+describe :: Monad m => BinPkgName -> DebT m Text
+describe b =
+    do Just p <- access T.packageDescription
+       typ <- access (T.packageType b)
+       return $
+          debianDescriptionBase p <> "\n" <>
+          case typ of
+            Just B.Profiling ->
+                Text.intercalate "\n"
+                        [" .",
+                         " This package provides a library for the Haskell programming language, compiled",
+                         " for profiling.  See http:///www.haskell.org/ for more information on Haskell."]
+            Just B.Development ->
+                Text.intercalate "\n"
+                        [" .",
+                         " This package provides a library for the Haskell programming language.",
+                         " See http:///www.haskell.org/ for more information on Haskell."]
+            Just B.Documentation ->
+                Text.intercalate "\n"
+                        [" .",
+                         " This package provides the documentation for a library for the Haskell",
+                         " programming language.",
+                         " See http:///www.haskell.org/ for more information on Haskell." ]
+            Just B.Exec ->
+                Text.intercalate "\n"
+                        [" .",
+                         " An executable built from the " <> pack (display (pkgName (Cabal.package p))) <> " package."]
+      {-    ServerPackage ->
+                Text.intercalate "\n"
+                        [" .",
+                         " A server built from the " <> pack (display (pkgName pkgId)) <> " package."] -}
+            _ {-Utilities-} ->
+                Text.intercalate "\n"
+                        [" .",
+                         " Files associated with the " <> pack (display (pkgName (Cabal.package p))) <> " package."]
+            -- x -> error $ "Unexpected library package name suffix: " ++ show x
 
 -- | The Cabal package has one synopsis and one description field
 -- for the entire package, while in a Debian package there is a
 -- description field (of which the first line is synopsis) in
 -- each binary package.  So the cabal description forms the base
 -- of the debian description, each of which is amended.
-debianDescriptionBase :: String -> String -> String -> String -> String -> Text
-debianDescriptionBase synopsis' description' author' maintainer' url =
-    (pack . unwords . words $ synopsis') <>
-    case description' of
+debianDescriptionBase :: PackageDescription -> Text
+debianDescriptionBase p =
+    (pack . unwords . words $ Cabal.synopsis p) <>
+    case Cabal.description p of
       "" -> ""
       text ->
           let text' = text ++ "\n" ++
-                      list "" ("\n Author: " ++) author' ++
-                      list "" ("\n Upstream-Maintainer: " ++) maintainer' ++
-                      list "" ("\n Url: " ++) url in
+                      list "" ("\n Author: " ++) (Cabal.author p) ++
+                      list "" ("\n Upstream-Maintainer: " ++) (Cabal.maintainer p) ++
+                      list "" ("\n Url: " ++) (Cabal.pkgUrl p) in
           "\n " <> (pack . trim . List.intercalate "\n " . List.map addDot . lines $ text')
     where
       addDot line = if all (flip elem " \t") line then "." else line
       list :: b -> ([a] -> b) -> [a] -> b
       list d f l = case l of [] -> d; _ -> f l
 
-oldClckwrksSiteFlags :: Site -> [String]
+oldClckwrksSiteFlags :: T.Site -> [String]
 oldClckwrksSiteFlags x =
     [ -- According to the happstack-server documentation this needs a trailing slash.
-      "--base-uri", "http://" ++ domain x ++ "/"
-    , "--http-port", show port]
-oldClckwrksServerFlags :: Server -> [String]
+      "--base-uri", "http://" ++ T.domain x ++ "/"
+    , "--http-port", show T.port]
+oldClckwrksServerFlags :: T.Server -> [String]
 oldClckwrksServerFlags x =
     [ -- According to the happstack-server documentation this needs a trailing slash.
-      "--base-uri", "http://" ++ hostname x ++ ":" ++ show (port x) ++ "/"
-    , "--http-port", show port]
+      "--base-uri", "http://" ++ T.hostname x ++ ":" ++ show (T.port x) ++ "/"
+    , "--http-port", show T.port]
 
 watchAtom :: PackageName -> Text
 watchAtom (PackageName pkgname) =
@@ -175,36 +169,40 @@ watchAtom (PackageName pkgname) =
            "-$1.tar.gz|\" \\\n    http://hackage.haskell.org/package/" ++ pkgname ++
            " \\\n    ([\\d\\.]*\\d)/\n"
 
-siteAtoms :: BinPkgName -> Site -> Atoms -> Atoms
+-- FIXME - use Atoms
+siteAtoms :: BinPkgName -> T.Site -> Atoms -> Atoms
 siteAtoms b site =
-    modL installDir (Map.insertWith Set.union b (singleton "/etc/apache2/sites-available")) .
-    modL link (Map.insertWith Set.union b (singleton ("/etc/apache2/sites-available/" ++ domain site, "/etc/apache2/sites-enabled/" ++ domain site))) .
-    modL file (Map.insertWith Set.union b (singleton ("/etc/apache2/sites-available" </> domain site, apacheConfig))) .
-    modL installDir (Map.insertWith Set.union b (singleton (apacheLogDirectory b))) .
-    modL logrotateStanza (Map.insertWith Set.union b (singleton (Text.unlines $
-                                                                 [ pack (apacheAccessLog b) <> " {"
-                                                                 , "  weekly"
-                                                                 , "  rotate 5"
-                                                                 , "  compress"
-                                                                 , "  missingok"
-                                                                 , "}"]))) .
-    modL logrotateStanza (Map.insertWith Set.union b (singleton (Text.unlines $
-                                                                 [ pack (apacheErrorLog b) <> " {"
-                                                                 , "  weekly"
-                                                                 , "  rotate 5"
-                                                                 , "  compress"
-                                                                 , "  missingok"
-                                                                 , "}" ]))) .
-    serverAtoms b (server site) True
+    execDebM
+      (do T.installDir +++= (b, singleton "/etc/apache2/sites-available")
+          T.link +++= (b, singleton ("/etc/apache2/sites-available/" ++ T.domain site, "/etc/apache2/sites-enabled/" ++ T.domain site))
+          T.file +++= (b, singleton ("/etc/apache2/sites-available" </> T.domain site, apacheConfig))
+          T.installDir +++= (b, singleton (apacheLogDirectory b))
+          T.logrotateStanza +++= (b, singleton
+                                   (Text.unlines $ [ pack (apacheAccessLog b) <> " {"
+                                                   , "  copytruncate" -- hslogger doesn't notice when the log is rotated, maybe this will help
+                                                   , "  weekly"
+                                                   , "  rotate 5"
+                                                   , "  compress"
+                                                   , "  missingok"
+                                                   , "}"]))
+          T.logrotateStanza +++= (b, singleton
+                                   (Text.unlines $ [ pack (apacheErrorLog b) <> " {"
+                                                   , "  copytruncate"
+                                                   , "  weekly"
+                                                   , "  rotate 5"
+                                                   , "  compress"
+                                                   , "  missingok"
+                                                   , "}" ]))) .
+      serverAtoms b (T.server site) True
     where
       -- An apache site configuration file.  This is installed via a line
       -- in debianFiles.
       apacheConfig =
           Text.unlines $
                    [  "<VirtualHost *:80>"
-                   , "    ServerAdmin " <> pack (serverAdmin site)
-                   , "    ServerName www." <> pack (domain site)
-                   , "    ServerAlias " <> pack (domain site)
+                   , "    ServerAdmin " <> pack (T.serverAdmin site)
+                   , "    ServerName www." <> pack (T.domain site)
+                   , "    ServerAlias " <> pack (T.domain site)
                    , ""
                    , "    ErrorLog " <> pack (apacheErrorLog b)
                    , "    CustomLog " <> pack (apacheAccessLog b) <> " combined"
@@ -233,33 +231,34 @@ siteAtoms b site =
                    , "    ProxyPass / http://127.0.0.1:" <> port' <> "/ nocanon"
                    , "    ProxyPassReverse / http://127.0.0.1:" <> port' <> "/"
                    , "</VirtualHost>" ]
-      port' = pack (show (port (server site)))
+      port' = pack (show (T.port (T.server site)))
 
-serverAtoms :: BinPkgName -> Server -> Bool -> Atoms -> Atoms
+-- FIXME - use Atoms
+serverAtoms :: BinPkgName -> T.Server -> Bool -> Atoms -> Atoms
 serverAtoms b server' isSite =
-    modL postInst (insertWith (error "serverAtoms") b debianPostinst) .
-    modL installInit (Map.insertWith (error "serverAtoms") b debianInit) .
+    modL T.postInst (insertWith (\ old new -> if old /= new then error ("serverAtoms: " ++ show old ++ " -> " ++ show new) else old) b debianPostinst) .
+    modL T.installInit (Map.insertWith (\ old new -> if old /= new then error ("serverAtoms: " ++ show old ++ " -> " ++ show new) else old) b debianInit) .
     serverLogrotate' b .
     execAtoms b exec
     where
-      exec = installFile server'
+      exec = T.installFile server'
       debianInit =
           Text.unlines $
                    [ "#! /bin/sh -e"
                    , ""
                    , ". /lib/lsb/init-functions"
-                   , "test -f /etc/default/" <> pack (destName exec) <> " && . /etc/default/" <> pack (destName exec)
+                   , "test -f /etc/default/" <> pack (T.destName exec) <> " && . /etc/default/" <> pack (T.destName exec)
                    , ""
                    , "case \"$1\" in"
                    , "  start)"
-                   , "    test -x /usr/bin/" <> pack (destName exec) <> " || exit 0"
-                   , "    log_begin_msg \"Starting " <> pack (destName exec) <> "...\""
+                   , "    test -x /usr/bin/" <> pack (T.destName exec) <> " || exit 0"
+                   , "    log_begin_msg \"Starting " <> pack (T.destName exec) <> "...\""
                    , "    mkdir -p " <> pack (databaseDirectory b)
                    , "    " <> startCommand
                    , "    log_end_msg $?"
                    , "    ;;"
                    , "  stop)"
-                   , "    log_begin_msg \"Stopping " <> pack (destName exec) <> "...\""
+                   , "    log_begin_msg \"Stopping " <> pack (T.destName exec) <> "...\""
                    , "    " <> stopCommand
                    , "    log_end_msg $?"
                    , "    ;;"
@@ -269,11 +268,11 @@ serverAtoms b server' isSite =
                    , "esac"
                    , ""
                    , "exit 0" ]
-      startCommand = pack $ showCommand "start-stop-daemon" (startOptions ++ commonOptions ++ ["--"] ++ serverFlags server')
+      startCommand = pack $ showCommand "start-stop-daemon" (startOptions ++ commonOptions ++ ["--"] ++ T.serverFlags server')
       stopCommand = pack $ showCommand "start-stop-daemon" (stopOptions ++ commonOptions)
-      commonOptions = ["--pidfile", "/var/run/" ++ destName exec]
-      startOptions = ["--start", "-b", "--make-pidfile", "-d", databaseDirectory b, "--exec", "/usr/bin" </> destName exec]
-      stopOptions = ["--stop", "--oknodo"] ++ if retry server' /= "" then ["--retry=" ++ retry server' ] else []
+      commonOptions = ["--pidfile", "/var/run/" ++ T.destName exec]
+      startOptions = ["--start", "-b", "--make-pidfile", "-d", databaseDirectory b, "--exec", "/usr/bin" </> T.destName exec]
+      stopOptions = ["--stop", "--oknodo"] ++ if T.retry server' /= "" then ["--retry=" ++ T.retry server' ] else []
 
       debianPostinst =
           Text.unlines $
@@ -300,24 +299,26 @@ serverAtoms b server' isSite =
 
 -- | A configuration file for the logrotate facility, installed via a line
 -- in debianFiles.
+-- FIXME - use Atoms
 serverLogrotate' :: BinPkgName -> Atoms -> Atoms
 serverLogrotate' b =
-    modL logrotateStanza (insertWith Set.union b (singleton (Text.unlines $ [ pack (serverAccessLog b) <> " {"
+    modL T.logrotateStanza (insertWith Set.union b (singleton (Text.unlines $ [ pack (serverAccessLog b) <> " {"
                                  , "  weekly"
                                  , "  rotate 5"
                                  , "  compress"
                                  , "  missingok"
                                  , "}" ]))) .
-    modL logrotateStanza (insertWith Set.union b (singleton (Text.unlines $ [ pack (serverAppLog b) <> " {"
+    modL T.logrotateStanza (insertWith Set.union b (singleton (Text.unlines $ [ pack (serverAppLog b) <> " {"
                                  , "  weekly"
                                  , "  rotate 5"
                                  , "  compress"
                                  , "  missingok"
                                  , "}" ])))
 
+-- FIXME - use Atoms
 backupAtoms :: BinPkgName -> String -> Atoms -> Atoms
 backupAtoms b name =
-    modL postInst (insertWith (error "backupAtoms") b
+    modL T.postInst (insertWith (\ old new -> if old /= new then error $ "backupAtoms: " ++ show old ++ " -> " ++ show new else old) b
                  (Text.unlines $
                   [ "#!/bin/sh"
                   , ""
@@ -326,27 +327,45 @@ backupAtoms b name =
                   , "    " <> pack ("/etc/cron.hourly" </> name) <> " --initialize"
                   , "    ;;"
                   , "esac" ])) .
-    execAtoms b (InstallFile { execName = name
-                              , destName = name
-                              , sourceDir = Nothing
-                              , destDir = Just "/etc/cron.hourly" })
+    execAtoms b (T.InstallFile { T.execName = name
+                               , T.destName = name
+                               , T.sourceDir = Nothing
+                               , T.destDir = Just "/etc/cron.hourly" })
 
-execAtoms :: BinPkgName -> InstallFile -> Atoms -> Atoms
+-- FIXME - use Atoms
+execAtoms :: BinPkgName -> T.InstallFile -> Atoms -> Atoms
 execAtoms b ifile r =
-    modL rulesFragments (Set.insert (pack ("build" </> show (pretty b) ++ ":: build-ghc-stamp"))) .
+    modL T.rulesFragments (Set.insert (pack ("build" </> show (pretty b) ++ ":: build-$(HC)-stamp"))) .
     fileAtoms b ifile $
     r
 
-fileAtoms :: BinPkgName -> InstallFile -> Atoms -> Atoms
+-- FIXME - use Atoms
+fileAtoms :: BinPkgName -> T.InstallFile -> Atoms -> Atoms
 fileAtoms b installFile' r =
-    fileAtoms' b (sourceDir installFile') (execName installFile') (destDir installFile') (destName installFile') r
+    fileAtoms' b (T.sourceDir installFile') (T.execName installFile') (T.destDir installFile') (T.destName installFile') r
 
+-- FIXME - use Atoms
 fileAtoms' :: BinPkgName -> Maybe FilePath -> String -> Maybe FilePath -> String -> Atoms -> Atoms
 fileAtoms' b sourceDir' execName' destDir' destName' r =
     case (sourceDir', execName' == destName') of
-      (Nothing, True) -> modL installCabalExec (insertWith Set.union b (singleton (execName', d))) r
-      (Just s, True) -> modL install (insertWith Set.union b (singleton (s </> execName', d))) r
-      (Nothing, False) -> modL installCabalExecTo (insertWith Set.union b (singleton (execName', (d </> destName')))) r
-      (Just s, False) -> modL installTo (insertWith Set.union b (singleton (s </> execName', d </> destName'))) r
+      (Nothing, True) -> modL T.installCabalExec (insertWith Set.union b (singleton (execName', d))) r
+      (Just s, True) -> modL T.install (insertWith Set.union b (singleton (s </> execName', d))) r
+      (Nothing, False) -> modL T.installCabalExecTo (insertWith Set.union b (singleton (execName', (d </> destName')))) r
+      (Just s, False) -> modL T.installTo (insertWith Set.union b (singleton (s </> execName', d </> destName'))) r
     where
       d = fromMaybe "usr/bin" destDir'
+
+-- | Build a suitable value for the head of the rules file.
+makeRulesHead :: Monad m => DebT m Text
+makeRulesHead =
+    do b <- debianName B.Cabal
+       hc <- access T.compilerFlavor
+       let ls = ["DEB_CABAL_PACKAGE = " <> pack (show (pretty (b :: BinPkgName))),
+                 "HC = " <> pack (map toLower (show hc)),
+                 ""]
+       return $
+          Text.unlines $
+            ["#!/usr/bin/make -f", ""] ++
+            ls ++
+            ["include /usr/share/cdbs/1/rules/debhelper.mk",
+             "include /usr/share/cdbs/1/class/hlibrary.mk"]
